@@ -1100,6 +1100,107 @@ class InsertQuery(BaseQuery):
         return self.database.last_insert_id(result, self.model)
 
 
+def convert_lookup(model, joins, lookup):
+    operations = model._meta.database.adapter.operations
+    
+    pieces = lookup.split('__')
+    operation = None
+    
+    query_model = model
+    
+    if len(pieces) > 1:
+        if pieces[-1] in operations:
+            operation = pieces.pop()
+        
+        lookup = pieces.pop()
+        
+        # we have some joins
+        if len(pieces):
+            query_model = model
+            
+            for piece in pieces:
+                # piece is something like 'blog' or 'entry_set'
+                joined_model = None
+                for field in query_model._meta.get_fields():
+                    if not isinstance(field, ForeignKeyField):
+                        continue
+                    
+                    if piece in (field.descriptor, field.related_name):
+                        joined_model = field.to
+                
+                if not joined_model:
+                    try:
+                        joined_model = query_model._meta.reverse_relations[piece]
+                    except KeyError:
+                        raise ValueError('Unknown relation: "%s" of "%s"' % (
+                            piece,
+                            query_model,
+                        ))
+                
+                joins.setdefault(query_model, set())
+                joins[query_model].add(joined_model)
+                query_model = joined_model
+    
+    if operation:
+        lookup = '%s__%s' % (lookup, operation)
+    
+    return query_model, joins, lookup
+
+
+def filter_query(model, *args, **kwargs):
+    """
+    Provide a django-like interface for executing queries
+    """
+    query = {} # mapping of models to queries
+    joins = {} # a graph of joins needed, passed into the convert_lookup function
+    
+    # due to quirks in the way where clauses are defined, Q() queries can only
+    # work on a single table -- this will need to be redesigned so Q() objects
+    # can work on multiple models
+    def fix_q(node_or_q, joins):
+        if isinstance(node_or_q, Node):
+            for child in node_or_q.children:
+                query_model = fix_q(child, joins)
+        elif isinstance(node_or_q, Q):
+            new_query = {}
+            for raw_lookup, value in node_or_q.query.items():
+                # do we have a query model at this point?
+                query_model, joins, lookup = convert_lookup(model, joins, raw_lookup)
+                new_query[lookup] = value
+            node_or_q.query = new_query
+        return query_model
+    
+    for node_or_q in args:
+        queried_model = fix_q(node_or_q, joins)
+        query.setdefault(queried_model, [])
+        query[queried_model].append(node_or_q)
+    
+    # iterate over keyword lookups and determine lookups and necessary joins
+    for raw_lookup, value in kwargs.items():
+        queried_model, joins, lookup = convert_lookup(model, joins, raw_lookup)
+        query.setdefault(queried_model, [])
+        query[queried_model].append((lookup, value))
+    
+    def follow_joins(current, query):
+        if current in joins:
+            for joined_model in joins[current]:
+                query = query.join(joined_model)
+                query = follow_joins(joined_model, query)
+        return query
+    select_query = follow_joins(model, model.select())
+    
+    for model, lookups in query.items():
+        qargs, qkwargs = [], {}
+        for lookup in lookups:
+            if isinstance(lookup, tuple):
+                qkwargs[lookup[0]] = lookup[1]
+            else:
+                qargs.append(lookup)
+        select_query = select_query.switch(model).where(*qargs, **qkwargs)
+
+    return select_query
+
+
 class Field(object):
     db_field = ''
     default = None
@@ -1110,12 +1211,15 @@ class Field(object):
     def get_attributes(self):
         return {}
     
-    def __init__(self, null=False, db_index=False, verbose_name=None, *args, **kwargs):
+    def __init__(self, null=False, db_index=False, unique=False, verbose_name=None,
+                 help_text=None, *args, **kwargs):
         self.null = null
         self.db_index = db_index
+        self.unique = unique
         self.attributes = self.get_attributes()
         self.default = kwargs.get('default', None)
         self.verbose_name = verbose_name
+        self.help_text = help_text
         
         kwargs['nullable'] = ternary(self.null, '', ' NOT NULL')
         self.attributes.update(kwargs)
@@ -1298,6 +1402,7 @@ class ForeignKeyField(IntegerField):
         
         reverse_rel = ReverseForeignRelatedObject(klass, self.name)
         setattr(self.to, self.related_name, reverse_rel)
+        self.to._meta.reverse_relations[self.related_name] = klass
     
     def lookup_value(self, lookup_type, value):
         if isinstance(value, Model):
@@ -1322,6 +1427,7 @@ class BaseModelOptions(object):
             setattr(self, k, v)
         
         self.rel_fields = {}
+        self.reverse_relations = {}
         self.fields = {}
         self.model_class = model_class
     
@@ -1455,14 +1561,18 @@ class Model(object):
             if isinstance(field_obj, PrimaryKeyField):
                 cls._meta.database.create_index(cls, field_obj.name, True)
             elif isinstance(field_obj, ForeignKeyField):
-                cls._meta.database.create_index(cls, field_obj.name)
-            elif field_obj.db_index:
-                cls._meta.database.create_index(cls, field_obj.name)
+                cls._meta.database.create_index(cls, field_obj.name, field_obj.unique)
+            elif field_obj.db_index or field_obj.unique:
+                cls._meta.database.create_index(cls, field_obj.name, field_obj.unique)
     
     @classmethod
     def drop_table(cls, fail_silently=False):
         cls._meta.database.drop_table(cls, fail_silently)
     
+    @classmethod
+    def filter(cls, *args, **kwargs):
+        return filter_query(cls, *args, **kwargs)
+    
     @classmethod
     def select(cls, query=None):
         return SelectQuery(cls, query)
